/*
 * Copyright 2019, Cypress Semiconductor Corporation or a subsidiary of
 * Cypress Semiconductor Corporation. All Rights Reserved.
 *
 * This software, including source code, documentation and related
 * materials ("Software"), is owned by Cypress Semiconductor Corporation
 * or one of its subsidiaries ("Cypress") and is protected by and subject to
 * worldwide patent protection (United States and foreign),
 * United States copyright laws and international treaty provisions.
 * Therefore, you may use this Software only as provided in the license
 * agreement accompanying the software package from which you
 * obtained this Software ("EULA").
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive,
 * non-transferable license to copy, modify, and compile the Software
 * source code solely for use in connection with Cypress's
 * integrated circuit products. Any reproduction, modification, translation,
 * compilation, or representation of this Software except as specified
 * above is prohibited without the express written permission of Cypress.
 *
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress
 * reserves the right to make changes to the Software without notice. Cypress
 * does not assume any liability arising out of the application or use of the
 * Software or any product or circuit described in the Software. Cypress does
 * not authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death ("High Risk Product"). By
 * including Cypress's product in a High Risk Product, the manufacturer
 * of such system or application assumes all risk of such use and in doing
 * so agrees to indemnify Cypress against all liability.
 */

/** app_bt_cfg.h
 *
 * Header File for runtime Bluetooth stack configuration parameters
 *
 */

#ifndef LOW_POWER_20719_H_
#define LOW_POWER_20719_H_

/*******************************************************************
 * Constant Definitions
 ******************************************************************/
/* Pass 10000 to HID-Off API to sleep for 10 seconds */
#define HIDOFF_SLEEP_TIME      10000
#define NOTFICATION_TIME_MS    5000     /* 5 seconds interval for notification if
                                           notifications enabled by client*/

#define BATTERY_LEVEL_FULL     100      /* Value of battery when full */
#define BATTERY_LEVEL_EMPTY    0        /* Value of battery when empty */

#define CONNECTION_INTERVAL    80       /* Connection interval of 100 ms */
#define SLAVE_LATENCY          0        /* Slave latency of 0 */
#define SUPERVISION_TIMEOUT    300      /* link Supervision timeout of 30 seconds */
/*******************************************************************
 * Variable Definitions
 ******************************************************************/

extern uint8_t send_notification_after_sds;

/*******************************************************************
 * Function Prototypes
 ******************************************************************/
static uint32_t app_bt_sleep_handler(wiced_sleep_poll_type_t type);
void            app_bt_enter_hid_off(void);

extern wiced_bt_dev_status_t  app_bt_management_callback
        (wiced_bt_management_evt_t event,
        wiced_bt_management_evt_data_t *p_event_data);

/* enum for system state */
extern enum
{
    SLEEP_WITHOUT_BLE,
    SLEEP_WITH_ADV,
    SLEEP_WITH_CONNECTION,
}application_state;

extern uint8_t app_bt_current_state;     /* Maintains the current system state */
extern uint64_t notification_timer_start_instance;
extern OSAPI_TIMER app_bt_notification_timer_handle;

void app_bt_timeout(int32_t count, uint32_t overTimeInUs);

#endif /* LOW_POWER_20719_H_ */
